import pandas as pd
import numpy as np
from pandas.api.types import is_numeric_dtype

class LabelEncodingDict():
    column_value_to_replace = None
    column_value_to_replace_with = None
    def __init__(self,column_value_to_replace, column_value_to_replace_with) -> None:
        self.column_value_to_replace = column_value_to_replace
        self.column_value_to_replace_with = column_value_to_replace_with

class PreLabelEncoderConfig():
    column_name = None
    label_encoding:list[LabelEncodingDict] = None
    def __init__(self, column_name,label_encoding):
        self.column_name = column_name
        self.label_encoding = label_encoding
    def convert_to_dict(self):
        return dict((obj.column_value_to_replace, obj.column_value_to_replace_with) for obj in self.label_encoding)
        

class PreNumericColDataChangeConfig():
    column_name = None
    data_type = None
    def __init__(self,col_name,data_type) -> None:
        self.column_name = col_name
        self.data_type = data_type

class PreProcessingConfig():
    encoding_dummies:list[str]
    label_encode:list[PreLabelEncoderConfig]
    numeric_cols_data_changer:list[PreNumericColDataChangeConfig]
    target_column:str
    exclude_columns:list[str]
    is_classification = False


    def __init__(
            self
            ,encoding_dummies:list[str]
            ,label_encode:list[PreLabelEncoderConfig]
            ,numeric_cols_data_changer:list[PreNumericColDataChangeConfig]
            ,exclude_columns
            ,target_column = None
            ,is_classification = False
            ):
        self.encoding_dummies = encoding_dummies
        self.label_encode = label_encode
        self.numeric_cols_data_changer = numeric_cols_data_changer
        self.target_column = target_column
        self.exclude_columns = exclude_columns
        self.is_classification = is_classification

def fillna(df):
    print('Filling NA values with median and mode for numeria and non-numeric columns.')
    df_internal = pd.DataFrame(df)
    for col in df_internal.columns:
        if(is_numeric_dtype(df_internal[col])):
            df_internal[col] = df_internal[col].fillna(df_internal[col].median())
        else:
            df_internal[col] = df_internal[col].fillna(df_internal[col].value_counts().idxmax())
    return df_internal


def remove_duplicates(df, subset=None, keep='first'):
    print('Removing duplicates.')
    df_unique = df.drop_duplicates(subset=subset, keep=keep)
    return df_unique



def null_unsuable_values_cleaner(frame, dirty_symbols_with_replacer_value=[{' ': np.nan}, {'?': np.nan}]):
    print('Removing unusable values like " " and "?"')
    df = pd.DataFrame(frame)
    for x in dirty_symbols_with_replacer_value:
        df = df.replace(x)
    return df


def numeric_columns_data_changer(df,cols_and_datatype:list[PreNumericColDataChangeConfig]):
    print('Changing datatype of numeric columns.')
    df_internal = pd.DataFrame(df)
    for x in cols_and_datatype:
        df_internal[x.column_name] = df_internal[x.column_name].astype(x.data_type)
    return df_internal

def label_encoding(df,columns_with_label_encoding:list[PreLabelEncoderConfig]):
    print('Performing label encoding on columns provided.')
    df_internal = pd.DataFrame(df)
    for x in columns_with_label_encoding:
        df_internal[x.column_name] =  df_internal[x.column_name].replace(x.convert_to_dict())
    return df_internal

def encoding_dummies(df,columns_to_dummise):
    print('Encoding using dummies.')
    df_internal = pd.DataFrame(df)
    df_internal = pd.get_dummies(df_internal, columns=columns_to_dummise)
    return df_internal

def handle_outliers_iqr(df,target_col, iqr_multiplier=1.5):
    print(f'Performing IQR based outliers cleaning on target. {target_col}')
    df_outliers_removed = df.copy()
    q1 = df[target_col].quantile(0.25)
    q3 = df[target_col].quantile(0.75)
    iqr = q3 - q1
    lower_bound = q1 - (iqr_multiplier * iqr)
    upper_bound = q3 + (iqr_multiplier * iqr)

    df_outliers_removed = df_outliers_removed[(df_outliers_removed[target_col] >= lower_bound) & (df_outliers_removed[target_col] <= upper_bound)]

    return df_outliers_removed

def process(df:pd.DataFrame,model_config:PreProcessingConfig):

    df_preprocessing = dropping_cols_rows(df)
    df_preprocessing = df_preprocessing.drop(columns=model_config.exclude_columns)
    df_preprocessing = null_unsuable_values_cleaner(df_preprocessing)
    df_preprocessing = fillna(df_preprocessing)
    if(model_config.is_classification == False):
        df_preprocessing = handle_outliers_iqr(df_preprocessing,model_config.target_column)
    df_preprocessing = numeric_columns_data_changer(df_preprocessing, model_config.numeric_cols_data_changer)
    df_preprocessing = label_encoding(df_preprocessing,model_config.label_encode)
    df_preprocessing = encoding_dummies(df_preprocessing, model_config.encoding_dummies)
    df_preprocessing = df_preprocessing[[col for col in df_preprocessing.columns if col != model_config.target_column] + [model_config.target_column]]
    return df_preprocessing.reset_index(drop=True)

def process_test(df:pd.DataFrame,model_config:PreProcessingConfig):

    df_preprocessing = dropping_cols_rows(df)
    df_preprocessing = df_preprocessing.drop(columns=model_config.exclude_columns)
    df_preprocessing = null_unsuable_values_cleaner(df_preprocessing)
    df_preprocessing = fillna(df_preprocessing)
    # df_preprocessing = handle_outliers_iqr(df_preprocessing,model_config.target_column)
    df_preprocessing = numeric_columns_data_changer(df_preprocessing, model_config.numeric_cols_data_changer)
    df_preprocessing = label_encoding(df_preprocessing,model_config.label_encode)
    df_preprocessing = encoding_dummies(df_preprocessing, model_config.encoding_dummies)
    # df_preprocessing = df_preprocessing[[col for col in df_preprocessing.columns if col != model_config.target_column] + [model_config.target_column]]
    return df_preprocessing.reset_index(drop=True)

def dropping_cols_rows(df,threshold = .7):
    print(f'Dropping columns whose values are nullable greater than {threshold}')
    df_internal = pd.DataFrame(df)
    for col in df_internal.columns:
        df_internal = df_internal[df_internal.columns[df_internal.isnull().mean() < threshold]]

        df_internal = df_internal.loc[df_internal.isnull().mean(axis=1) < threshold]
    return df_internal







import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.ensemble import RandomForestClassifier
from sklearn.tree import DecisionTreeClassifier
from sklearn.linear_model import Lasso
from auto_ml_kinder import model_list_helper as mlh
from sklearn.model_selection import RandomizedSearchCV


def feature_importance_end_model_builder(feature_importance_dataframe,columns):
    beautified_df = pd.DataFrame({'column_name':columns,'value':feature_importance_dataframe[0]})
    return beautified_df

def run_feature_selection(df):
    print('Executing run_feature_selection which runs RandomForest, DecisionTree and Lasso models to find the features used in the model training.')
    important_features = pd.concat([RandomForestFeaturesSelection(df),DecisionTreeClassifierFeaturesSelection(df),LassoFeaturesSelection(df)])
    # print(important_features)
    important_features_non_duplicated = important_features[important_features.column_name.duplicated() == False]
    signal_df_processed_important_features = df.iloc[:,important_features_non_duplicated.column_name.to_numpy()]
    signal_df_processed_important_features[df.columns[len(df.columns)-1]] = df.iloc[:,-1]
    return signal_df_processed_important_features

def RandomForestFeaturesSelection(X:pd.DataFrame,Y:pd.Series):
    print('Executing RandomForestFeaturesSelection function to get the best features that are used in training the model.')
    model = RandomSearch_Selector(*mlh.get_parameters_random_forest_fit_reg(mlh.ModelPower.HIGH),X_train=X,y_train=Y)
    model.fit(X, Y)
    feature_importance = model.feature_importances_
    feature_importance_dataframe = pd.DataFrame(feature_importance)
    beautified_df = feature_importance_end_model_builder(feature_importance_dataframe,X.columns)
    comparer_get_more_than = beautified_df['value'].mean()
    temp2 = beautified_df[beautified_df['value'] > comparer_get_more_than].sort_values(by='value',ascending=False).reset_index(drop='index')
    return temp2.column_name.values

def DecisionTreeClassifierFeaturesSelection(X:pd.DataFrame,Y:pd.Series):
    print('Executing DecisionTreeClassifierFeaturesSelection function to get the best features that are used in training the model.')
    model = RandomSearch_Selector(*mlh.get_parameters_decision_tree_fit_reg(mlh.ModelPower.HIGH),X_train=X,y_train=Y)
    model.fit(X, Y)
    feature_importance = model.feature_importances_
    feature_importance_dataframe = pd.DataFrame(feature_importance)
    beautified_df = feature_importance_end_model_builder(feature_importance_dataframe,X.columns)
    comparer_get_more_than = beautified_df['value'].mean()
    temp2 = beautified_df[beautified_df['value'] > comparer_get_more_than].sort_values(by='value',ascending=False).reset_index(drop='index')
    return temp2.column_name.values

def LassoFeaturesSelection(X:pd.DataFrame,Y:pd.Series):
    print('Executing LassoFeaturesSelection function to get the best features that are used in training the model.')
    model = RandomSearch_Selector(*mlh.get_parameters_lasso_fit(mlh.ModelPower.HIGH),X_train=X,y_train=Y)
    model.fit(X, Y)
    feature_importance = model.coef_
    feature_importance_dataframe = pd.DataFrame(feature_importance)
    beautified_df = feature_importance_end_model_builder(feature_importance_dataframe,X.columns)
    temp2 = beautified_df[beautified_df['value'] > 0].sort_values(by='value',ascending=False).reset_index(drop='index')
    return temp2.column_name.values

def RandomSearch_Selector(param_distributions,model, X_train, y_train, scoring='r2', cv=5):
    total_combinations = 1
    for values in param_distributions.values():
        total_combinations *= len(values)
    # Set n_iter based on the total_combinations
    n_iter = min(total_combinations, 10)

    random_search = RandomizedSearchCV(estimator=model, param_distributions=param_distributions,n_iter=n_iter, scoring=scoring, cv=cv, random_state=42)
    random_search.fit(X_train, y_train)
    
    best_model = random_search.best_estimator_
    
    return best_model

def Run_Features_Selection(X:pd.DataFrame,Y:pd.Series):
    array1 = RandomForestFeaturesSelection(X,Y)
    array2 = DecisionTreeClassifierFeaturesSelection(X,Y)
    array3 = LassoFeaturesSelection(X,Y)
    combined_array = []

    combined_array.extend(array1)
    combined_array.extend(array2)
    combined_array.extend(array3)

    distinct_values = set(combined_array)

    distinct_values_list = list(distinct_values)
    return distinct_values_list






from sklearn.ensemble import RandomForestRegressor
from sklearn.tree import DecisionTreeRegressor
from sklearn.svm import SVR
from sklearn.linear_model import ElasticNet
from sklearn.ensemble import GradientBoostingRegressor
from enum import Enum
from sklearn.linear_model import LinearRegression
from sklearn.linear_model import Ridge
from sklearn.linear_model import Lasso
from sklearn.neighbors import KNeighborsRegressor
from auto_ml_kinder.classes.ModelPower import ModelPower

def get_parameters_linear_reg():
    linear_reg_hyper_params = {
        'fit_intercept': [True, False]
    }
    # print(f'LinearRegression params: {linear_reg_hyper_params}')
    return linear_reg_hyper_params, LinearRegression()

def get_parameters_ridge_fit(power:ModelPower):
    alpha_options = [0.1, 0.5, 1.0, 2.0, 5.0, 10.0, 20.0]
    if power == ModelPower.LOW:
        alpha_options = [0.1, 0.5, 1.0]
    elif power == ModelPower.MEDIUM:
        alpha_options = [0.1, 0.5, 1.0, 2.0, 5.0]
    elif power == ModelPower.LITE:
        alpha_options = [0.1, 0.5]
    elif power == ModelPower.HIGH:
        alpha_options = [0.1, 0.5, 1.0, 2.0, 5.0, 10.0, 20.0, 50.0, 100.0]

    ridge_hyper_params = {
        'alpha': alpha_options,
        'solver': ['auto', 'svd', 'cholesky', 'lsqr', 'sparse_cg', 'sag', 'saga']
    }
    # print(f'Ridge params: {ridge_hyper_params}')
    return ridge_hyper_params, Ridge()

def get_parameters_lasso_fit(power):
    alpha_options = [0.1, 0.5, 1.0, 2.0, 5.0, 10.0, 20.0]
    if power == ModelPower.LOW:
        alpha_options = [0.1, 0.5, 1.0]
    elif power == ModelPower.MEDIUM:
        alpha_options = [0.1, 0.5, 1.0, 2.0, 5.0]
    elif power == ModelPower.LITE:
        alpha_options = [0.1, 0.5]
    elif power == ModelPower.HIGH:
        alpha_options = [0.1, 0.5, 1.0, 2.0, 5.0, 10.0, 20.0, 50.0, 100.0]

    lasso_hyper_params = {
        'alpha': alpha_options,
        'selection': ['cyclic', 'random']
    }
    # print(f'Lasso params: {lasso_hyper_params}')
    return lasso_hyper_params, Lasso()

def get_parameters_elasticnet_fit(power):
    alpha_options = [0.1, 0.5, 1.0, 2.0, 5.0, 10.0, 20.0]
    if power == ModelPower.LOW:
        alpha_options = [0.1, 0.5, 1.0]
    elif power == ModelPower.MEDIUM:
        alpha_options = [0.1, 0.5, 1.0, 2.0, 5.0]
    elif power == ModelPower.LITE:
        alpha_options = [0.1, 0.5]
    elif power == ModelPower.HIGH:
        alpha_options = [0.1, 0.5, 1.0, 2.0, 5.0, 10.0, 20.0, 50.0, 100.0]

    l1_ratio_options = [0.1, 0.3, 0.5, 0.7, 0.9]
    elasticnet_hyper_params = {
        'alpha': alpha_options,
        'l1_ratio': l1_ratio_options,
        # 'normalize': [True, False],
        'selection': ['cyclic', 'random']
    }
    # print(f'ElasticNet params: {elasticnet_hyper_params}')
    return elasticnet_hyper_params, ElasticNet()

def get_parameters_svr_fit(power):
    kernel_options = ['linear', 'poly', 'rbf', 'sigmoid']
    if power == ModelPower.LOW:
        C_options = [0.1, 1.0]
        epsilon_options = [0.1, 0.5]
    elif power == ModelPower.MEDIUM:
        C_options = [0.1, 1.0, 10.0]
        epsilon_options = [0.1, 0.5, 1.0]
    elif power == ModelPower.LITE:
        C_options = [0.1, 1.0]
        epsilon_options = [0.1]
    elif power == ModelPower.HIGH:
        C_options = [0.1, 1.0, 10.0, 100.0]
        epsilon_options = [0.1, 0.5, 1.0, 2.0]

    svr_hyper_params = {
        'kernel': kernel_options,
        'C': C_options,
        'epsilon': epsilon_options,
        'gamma': ['scale', 'auto']
    }
    # print(f'SVR params: {svr_hyper_params}')
    return svr_hyper_params, SVR()

def get_parameters_decision_tree_fit_reg(power):
    max_depth_options = [None, 5, 10, 20]
    min_samples_split_options = [2, 5, 10]
    if power == ModelPower.LOW:
        max_depth_options = [None, 5]
        min_samples_split_options = [2, 5]
    elif power == ModelPower.MEDIUM:
        max_depth_options = [None, 5, 10]
        min_samples_split_options = [2, 5, 10]
    elif power == ModelPower.LITE:
        max_depth_options = [None]
        min_samples_split_options = [2]
    elif power == ModelPower.HIGH:
        max_depth_options = [None, 5, 10, 20, 50]
        min_samples_split_options = [2, 5, 10, 20]

    decision_tree_hyper_params = {
        'max_depth': max_depth_options,
        'min_samples_split': min_samples_split_options,
        'min_samples_leaf': [1, 2, 4],
        'max_features': ['sqrt', 'log2']
    }
    # print(f'DecisionTree params: {decision_tree_hyper_params}')
    return decision_tree_hyper_params, DecisionTreeRegressor()

def get_parameters_random_forest_fit_reg(power):
    n_estimators_options = [50, 100, 200]
    max_depth_options = [None, 5, 10, 20]
    min_samples_split_options = [2, 5, 10]
    if power == ModelPower.LOW:
        n_estimators_options = [50, 100]
        max_depth_options = [None, 5]
        min_samples_split_options = [2, 5]
    elif power == ModelPower.MEDIUM:
        n_estimators_options = [50, 100, 200]
        max_depth_options = [None, 5, 10]
        min_samples_split_options = [2, 5, 10]
    elif power == ModelPower.LITE:
        n_estimators_options = [50]
        max_depth_options = [None]
        min_samples_split_options = [2]
    elif power == ModelPower.HIGH:
        n_estimators_options = [50, 100, 200, 500]
        max_depth_options = [None, 5, 10, 20]
        min_samples_split_options = [2, 5, 10, 20]

    random_forest_hyper_params = {
        'n_estimators': n_estimators_options,
        'max_depth': max_depth_options,
        'min_samples_split': min_samples_split_options,
        'min_samples_leaf': [1, 2, 4],
        'max_features': ['sqrt', 'log2']
    }
    # print(f'RandomForest params: {random_forest_hyper_params}')
    return random_forest_hyper_params, RandomForestRegressor()

def get_parameters_gradient_boosting_fit_reg(power):
    n_estimators_options = [50, 100, 200]
    learning_rate_options = [0.01, 0.1, 0.5]
    max_depth_options = [3, 5, 7]
    min_samples_split_options = [2, 5, 10]
    if power == ModelPower.LOW:
        n_estimators_options = [50, 100]
        learning_rate_options = [0.01, 0.1]
        max_depth_options = [3, 5]
        min_samples_split_options = [2, 5]
    elif power == ModelPower.MEDIUM:
        n_estimators_options = [50, 100, 200]
        learning_rate_options = [0.01, 0.1, 0.5]
        max_depth_options = [3, 5, 7]
        min_samples_split_options = [2, 5, 10]
    elif power == ModelPower.LITE:
        n_estimators_options = [50]
        learning_rate_options = [0.01]
        max_depth_options = [3]
        min_samples_split_options = [2]
    elif power == ModelPower.HIGH:
        n_estimators_options = [50, 100, 200, 500]
        learning_rate_options = [0.01, 0.1, 0.5]
        max_depth_options = [3, 5, 7, 10]
        min_samples_split_options = [2, 5, 10, 20]

    gradient_boosting_hyper_params = {
        'n_estimators': n_estimators_options,
        'learning_rate': learning_rate_options,
        'max_depth': max_depth_options,
        'min_samples_split': min_samples_split_options,
        'min_samples_leaf': [1, 2, 4],
        'max_features': ['auto', 'sqrt', 'log2']
    }
    # print(f'GradientBoosting params: {gradient_boosting_hyper_params}')
    return gradient_boosting_hyper_params, GradientBoostingRegressor()

def get_parameters_knn_reg(power):
    n_neighbors_values = [3, 5, 7, 10, 15]
    weights_options = ['uniform', 'distance']
    
    if power == ModelPower.LOW:
        n_neighbors_values = [3, 5, 7]
    elif power == ModelPower.MEDIUM:
        n_neighbors_values = [3, 5, 7, 10]
    elif power == ModelPower.LITE:
        n_neighbors_values = [3, 5]
    elif power == ModelPower.HIGH:
        n_neighbors_values = [3, 5, 7, 10, 15]

    knn_reg_hyper_params = {
        'n_neighbors': n_neighbors_values,
        'weights': weights_options
    }
    # print(f'KNN params: {knn_reg_hyper_params}')
    return knn_reg_hyper_params, KNeighborsRegressor()


from sklearn.ensemble import RandomForestClassifier, GradientBoostingClassifier
from sklearn.tree import DecisionTreeClassifier
from sklearn.svm import SVC
from sklearn.linear_model import LogisticRegression, RidgeClassifier
from sklearn.neighbors import KNeighborsClassifier
from enum import Enum

def get_parameters_logistic_reg():
    logistic_reg_hyper_params = {
        'penalty': ['l1', 'l2', 'elasticnet', 'none'],
        'C': [0.01, 0.1, 1.0, 10.0, 100.0],
        'solver': ['liblinear', 'saga', 'lbfgs', 'newton-cg']
    }
    return logistic_reg_hyper_params, LogisticRegression()

def get_parameters_ridge_classifier(power: ModelPower):
    alpha_options = [0.1, 0.5, 1.0, 2.0, 5.0, 10.0, 20.0]
    if power == ModelPower.LOW:
        alpha_options = [0.1, 0.5, 1.0]
    elif power == ModelPower.MEDIUM:
        alpha_options = [0.1, 0.5, 1.0, 2.0, 5.0]
    elif power == ModelPower.LITE:
        alpha_options = [0.1, 0.5]
    elif power == ModelPower.HIGH:
        alpha_options = [0.1, 0.5, 1.0, 2.0, 5.0, 10.0, 20.0, 50.0, 100.0]

    ridge_classifier_hyper_params = {
        'alpha': alpha_options,
        'solver': ['auto', 'svd', 'cholesky', 'lsqr', 'sparse_cg', 'sag', 'saga']
    }
    return ridge_classifier_hyper_params, RidgeClassifier()

def get_parameters_svc_fit(power: ModelPower):
    kernel_options = ['linear', 'poly', 'rbf', 'sigmoid']
    if power == ModelPower.LOW:
        C_options = [0.1, 1.0]
        gamma_options = ['scale']
    elif power == ModelPower.MEDIUM:
        C_options = [0.1, 1.0, 10.0]
        gamma_options = ['scale', 'auto']
    elif power == ModelPower.LITE:
        C_options = [0.1, 1.0]
        gamma_options = ['scale']
    elif power == ModelPower.HIGH:
        C_options = [0.1, 1.0, 10.0, 100.0]
        gamma_options = ['scale', 'auto']

    svc_hyper_params = {
        'kernel': kernel_options,
        'C': C_options,
        'gamma': gamma_options
    }
    return svc_hyper_params, SVC()

def get_parameters_decision_tree_classifier(power: ModelPower):
    max_depth_options = [None, 5, 10, 20]
    min_samples_split_options = [2, 5, 10]
    if power == ModelPower.LOW:
        max_depth_options = [None, 5]
        min_samples_split_options = [2, 5]
    elif power == ModelPower.MEDIUM:
        max_depth_options = [None, 5, 10]
        min_samples_split_options = [2, 5, 10]
    elif power == ModelPower.LITE:
        max_depth_options = [None]
        min_samples_split_options = [2]
    elif power == ModelPower.HIGH:
        max_depth_options = [None, 5, 10, 20, 50]
        min_samples_split_options = [2, 5, 10, 20]

    decision_tree_hyper_params = {
        'max_depth': max_depth_options,
        'min_samples_split': min_samples_split_options,
        'min_samples_leaf': [1, 2, 4],
        'max_features': ['sqrt', 'log2']
    }
    return decision_tree_hyper_params, DecisionTreeClassifier()

def get_parameters_random_forest_classifier(power: ModelPower):
    n_estimators_options = [50, 100, 200]
    max_depth_options = [None, 5, 10, 20]
    min_samples_split_options = [2, 5, 10]
    if power == ModelPower.LOW:
        n_estimators_options = [50, 100]
        max_depth_options = [None, 5]
        min_samples_split_options = [2, 5]
    elif power == ModelPower.MEDIUM:
        n_estimators_options = [50, 100, 200]
        max_depth_options = [None, 5, 10]
        min_samples_split_options = [2, 5, 10]
    elif power == ModelPower.LITE:
        n_estimators_options = [50]
        max_depth_options = [None]
        min_samples_split_options = [2]
    elif power == ModelPower.HIGH:
        n_estimators_options = [50, 100, 200, 500]
        max_depth_options = [None, 5, 10, 20]
        min_samples_split_options = [2, 5, 10, 20]

    random_forest_hyper_params = {
        'n_estimators': n_estimators_options,
        'max_depth': max_depth_options,
        'min_samples_split': min_samples_split_options,
        'min_samples_leaf': [1, 2, 4],
        'max_features': ['sqrt', 'log2']
    }
    return random_forest_hyper_params, RandomForestClassifier()

def get_parameters_gradient_boosting_classifier(power: ModelPower):
    n_estimators_options = [50, 100, 200]
    learning_rate_options = [0.01, 0.1, 0.5]
    max_depth_options = [3, 5, 7]
    min_samples_split_options = [2, 5, 10]
    if power == ModelPower.LOW:
        n_estimators_options = [50, 100]
        learning_rate_options = [0.01, 0.1]
        max_depth_options = [3, 5]
        min_samples_split_options = [2, 5]
    elif power == ModelPower.MEDIUM:
        n_estimators_options = [50, 100, 200]
        learning_rate_options = [0.01, 0.1, 0.5]
        max_depth_options = [3, 5, 7]
        min_samples_split_options = [2, 5, 10]
    elif power == ModelPower.LITE:
        n_estimators_options = [50]
        learning_rate_options = [0.01]
        max_depth_options = [3]
        min_samples_split_options = [2]
    elif power == ModelPower.HIGH:
        n_estimators_options = [50, 100, 200, 500]
        learning_rate_options = [0.01, 0.1, 0.5]
        max_depth_options = [3, 5, 7, 10]
        min_samples_split_options = [2, 5, 10, 20]

    gradient_boosting_hyper_params = {
        'n_estimators': n_estimators_options,
        'learning_rate': learning_rate_options,
        'max_depth': max_depth_options,
        'min_samples_split': min_samples_split_options,
        'min_samples_leaf': [1, 2, 4],
        'max_features': ['auto', 'sqrt', 'log2']
    }
    return gradient_boosting_hyper_params, GradientBoostingClassifier()

def get_parameters_knn_classifier(power: ModelPower):
    n_neighbors_values = [3, 5, 7, 10, 15]
    weights_options = ['uniform', 'distance']
    
    if power == ModelPower.LOW:
        n_neighbors_values = [3, 5, 7]
    elif power == ModelPower.MEDIUM:
        n_neighbors_values = [3, 5, 7, 10]
    elif power == ModelPower.LITE:
        n_neighbors_values = [3, 5]
    elif power == ModelPower.HIGH:
        n_neighbors_values = [3, 5, 7, 10, 15, 20]
    
    knn_hyper_params = {
        'n_neighbors': n_neighbors_values,
        'weights': weights_options
    }
    
    return knn_hyper_params, KNeighborsClassifier()


